// module ESP8266_ser.cpp
// Comm. program for ESP8266 WIFI module over serial line (simulation over USB)
// Can send a LUA file to the ESP8266 device
// This is for Windows 32 Bit
//
// Dr. E.Huckert, Goldbach(Germany) 2002-04-20, 2017-01

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <io.h>
#include <time.h>

#include "hu_rs232.h"

// global variables
int     logging     = 0;
char    protocol_name[] = "ESP8266_ser.txt";
char    start_verz[128] = ".\\";

// -----------------------------------------------------------------------------------------------
void usage()
{
  printf(
   "usage():ESP8266_ser [-file filename] [-baud baudrate] [-com no] [-v]\n");
  printf("baud rates: 9600,19200,38400,57600,115200\n");
  printf("Always start the receiver before the sender\n");
  printf("Copyright Dr.E.Huckert 2002/2017-01-09\n");
}

// -----------------------------------------------------------------------------------------------
int main(int argc, char *argv[])
{
int            n, ret;
char           buf[512];
int            receiver;
char           filename[128];
unsigned long  baud;
unsigned       comno;
HU_RS232       *pserial = NULL;
time_t         t1, t2;
FILE *ifd = NULL;
int cRet;
  //
  usage();
  ret      = 0;
  receiver = 0;
  comno    = 1;
  baud     = 9600L;
  strcpy(filename, "temp.txt");
  //
  for (n=1; n < argc; n++)
  {
    if (strcmp(argv[n], "-v") == 0)
      logging = 1;
    if (strcmp(argv[n], "-v2") == 0)
      logging = 2;
    if (strcmp(argv[n], "-baud") == 0)
      baud = atol(argv[n+1]);
    if (strcmp(argv[n], "-com") == 0)
      comno  = atoi(argv[n+1]);
    if (strcmp(argv[n], "-file") == 0)
      strcpy(filename, argv[n + 1]);
  }
  //
  printf("com=com%u\n", comno);
  printf("baud=%lu\n",baud);
  printf("file=%s\n\n", filename);
  //
  pserial = new HU_RS232(comno, baud);
  pserial->set_logging(logging);
  pserial->set_timeout(20);
  cRet = pserial->open_line(1);
  if (logging)
    printf("Ergebnis open_line()=%d\n", cRet);
  if (cRet < 0)
  {
    printf("ERROR: cannot open the serial device: COM%u\n", comno);
    printf("message=%s\n", pserial->getErrorString());
    ret = -1;
    goto zurueck;
  }
  ifd = ::fopen(filename, "r");
  if (ifd == NULL)
  {
    printf("ERROR: inp file not open\n");
    ret = -2;
    goto zurueck;
  }
  ::Sleep(200);
  char line[256];
  while (1)
  {
    if (::fgets(line, sizeof(line), ifd) == NULL)
      break;
    cRet = pserial->put_line(line, ::strlen(line));
    if (logging)
      printf("result put_line(): %d\n", cRet);
    if (cRet < 0)
    {
      ret = -3;
      goto zurueck;
    }
    printf("TX:%s", line);
    // allow max. 750 ms (10 * 75) for a reaction
    for (int n=0; n < 10; n++)
    {
      // get some kind of reaction
      cRet = pserial->read_line(line, sizeof(line) - 1);
      if (logging)
        printf("result read_line(): %d\n", cRet);
      if (cRet < 0)
        break;
      if (cRet > 0)
      {
        line[cRet] = 0; // is not 0-terminated in read_line()!
        printf("RX:%s", line);
        // Prompt character received?
        if (line[0] == '>')
          break;
      }
      // nothing received (yet)
      ::Sleep(75);
    }   // end for...
    // wait 25 ms after each transmission (send)
    ::Sleep(25);
  }   // end while (1)
  //
  ::fclose(ifd);
  ret = 1;
  //
  zurueck:
  if (pserial != NULL)
  {
    pserial->close_line();
    delete pserial;
  }
  //
  return ret;
}    // end main()

